<?php

$ConfigFileName = dirname(__FILE__).'/../Config/Config.php';
if(file_exists($ConfigFileName)) include_once($ConfigFileName);

include_once(dirname(__FILE__).'/Version.php');
include_once(dirname(__FILE__).'/../Common/Global.php');
include_once(dirname(__FILE__).'/FormClasses.php');
include_once(dirname(__FILE__).'/FullInstall.php');
include_once(dirname(__FILE__).'/UpdateTrace.php');
include_once(dirname(__FILE__).'/View.php');
include_once(dirname(__FILE__).'/DefaultConfig.php');

class Core extends Application
{
  /** @var Type */
  var $Type;
  var $Pages;
  var $Bars;
  /** @var FormManager */
  var $FormManager;
  /** @var Config */
  var $ConfigManager;
  var $PathItems;
  var $RootURLFolder;
  var $ShowPage;
  var $Setup;
  var $CommandLine;
  var $PageHeaders;
  var $BaseView;

  function __construct()
  {
    parent::__construct();
    $this->Modules = array();
    $this->Pages = array();
    $this->ModuleManager->FileName = dirname(__FILE__).'/../Config/ModulesConfig.php';
    $this->FormManager = new FormManager($this->Database);
    $this->ShowPage = true;
    $this->ConfigManager = new Config();
    $this->RootURLFolder = $_SERVER['SCRIPT_NAME'];
    if(substr($this->RootURLFolder, -10, 10) == '/index.php')
      $this->RootURLFolder = substr($this->RootURLFolder, 0, -10);
    $this->CommandLine = array();
    $this->PageHeaders = array();
  }

  function RegisterPage($Path, $Handler)
  {
    if(is_array($Path))
    {
      $Page = &$this->Pages;
      $LastKey = array_pop($Path);
      foreach($Path as $PathItem)
      {
        $Page = &$Page[$PathItem];
      }
      if(!is_array($Page)) $Page = array('' => $Page);
      $Page[$LastKey] = $Handler;
    } else $this->Pages[$Path] = $Handler;
  }

  function UnregisterPage($Path)
  {
    unset($this->Pages[$Path]);
  }

  function SearchPage($PathItems, $Pages)
  {
    if(count($PathItems) > 0) $PathItem = $PathItems[0];
      else $PathItem = '';
    if(array_key_exists($PathItem, $Pages))
    {
      if(is_array($Pages[$PathItem]))
      {
        array_shift($PathItems);
        return($this->SearchPage($PathItems, $Pages[$PathItem]));
      } else return($Pages[$PathItem]);
    } else return('');
  }

  function PageNotFound()
  {
    return('Page '.implode('/', $this->PathItems).' not found.');
  }

  function ShowPage()
  {
    $this->BaseView = new BaseView($this);

    /* @var $Page Page */
    $ClassName = $this->SearchPage($this->PathItems, $this->Pages);
    if($ClassName != '')
    {
      $Page = new $ClassName($this);
    } else {
      $Page = new PageMissing($this);
    }
    echo($this->BaseView->GetOutput($Page));
  }

  function ModulePresent($Name)
  {
    return(array_key_exists($Name, $this->Modules));
  }

  function AddModule($Module)
  {
    $this->Modules[get_class($Module)] = $Module;
  }

  function HumanDate($Time)
  {
    return(date('j.n.Y', $Time));
  }

  function Link($Target)
  {
    return($this->RootURLFolder.$Target);
  }

  function ShowAction($Id)
  {
    $Output = '';
    $DbResult = $this->Database->query('SELECT *, `ActionIcon`.`Name` AS `Icon` FROM `Action` '.
      'LEFT JOIN `ActionIcon` ON `ActionIcon`.`Id` = `Action`.`Icon` '.
      'WHERE (`Action`.`Id`='.$Id.')');
    if($DbResult->num_rows > 0)
    {
      $Action = $DbResult->fetch_assoc();
      if($Action['Icon'] == '') $Action['Icon'] = 'clear.png';
      if(substr($Action['URL'], 0, 4) != 'http') $Action['URL'] = $this->Link($Action['URL']);
      if(!defined('NEW_PERMISSION') or $this->User->CheckPermission('System', 'Read', 'Item', $Id))
        $Output .= '<img alt="'.$Action['Title'].'" src="'.$this->Link('/images/favicons/'.$Action['Icon']).
        '" width="16" height="16" /> <a href="'.$Action['URL'].'">'.$Action['Title'].'</a>';
    }
    return($Output);
  }

  function RunCommon()
  {
    global $Database, $ScriptTimeStart, $ConfigFileName, $Mail, $Type,
      $DatabaseRevision, $Config, $GlobalLocaleManager;

    date_default_timezone_set('Europe/Prague');
    mb_internal_encoding("UTF-8");
    $ScriptTimeStart = GetMicrotime();

    // SQL injection hack protection
    foreach($_POST as $Index => $Item) $_POST[$Index] = addslashes($Item);
    foreach($_GET as $Index => $Item) $_GET[$Index] = addslashes($Item);

    if(isset($_SERVER['REMOTE_ADDR'])) session_start();

    $ConfigFileName = dirname(dirname(__FILE__)).'/config.php';
    if(file_exists($ConfigFileName))
      $this->ConfigManager->LoadFromFile($ConfigFileName);
    //$this->Config = $this->ConfigManager->GetAsArray();
    $this->Config = &$Config;

    try {
      $this->Database->Connect($this->Config['Database']['Host'], $this->Config['Database']['User'],
        $this->Config['Database']['Password'], $this->Config['Database']['Database']);
      $this->Database->Prefix = $this->Config['Database']['Prefix'];
      $this->Database->charset($this->Config['Database']['Charset']);
      $this->Database->ShowSQLError = $this->Config['Web']['ShowSQLError'];
      $this->Database->ShowSQLQuery = $this->Config['Web']['ShowSQLQuery'];
      if(isset($this->Config['Web']['LogSQLQuery']))
        $this->Database->LogSQLQuery = $this->Config['Web']['LogSQLQuery'];
    } catch (Exception $E) {
      //$Output .= 'Nelze se připojit k databázi.';
    }

    $this->LocaleManager = new LocaleManager($this);
    $this->LocaleManager->Dir = dirname(dirname(__FILE__)).'/locale';
    $this->LocaleManager->Available = array(
      'cs' => array('Code' => 'cs', 'Title' => 'Česky'),
      'en' => array('Code' => 'en', 'Title' => 'English'),
    );
    $GlobalLocaleManager = $this->LocaleManager;
    $this->LinkLocaleExceptions = array('images', 'style', 'files');

    if(isset($this->Config['Web']['RootFolder']))
      $this->RootURLFolder = $this->Config['Web']['RootFolder'];
    $this->FormManager->Root = $this->RootURLFolder;

    $this->RegisterPageBar('Top');

    $Database = $this->Database;
    RegisterFormClasses($this->FormManager);

    // Register and start existing modules
    $this->Setup = new Setup($this);
    $this->Setup->Start();
    if($this->Setup->CheckState())
    {
      $this->ModuleManager->Start();
    }
  }

  function Run()
  {
    $this->RunCommon();
    if($this->ShowPage)
    {
      $this->PathItems = ProcessURL();

      // Detect interface locale
      if(isset($this->Config['Web']['Locale']))
        $this->LocaleManager->DefaultLangCode = $this->Config['Web']['Locale'];
      $this->LocaleManager->LangCode = $this->LocaleManager->DefaultLangCode;
      if(count($this->PathItems) > 0)
      {
         $NewLangCode = $this->PathItems[0];
         if(array_key_exists($NewLangCode, $this->LocaleManager->Available)) 
         {
           array_shift($this->PathItems);
           $this->LocaleManager->LangCode = $NewLangCode;
         }
      }
      if(array_key_exists($this->LocaleManager->LangCode, $this->LocaleManager->Available))
        $this->LocaleManager->LoadLocale($this->LocaleManager->LangCode);

      $this->ShowPage();
    }
  }

  function RunCommandLine()
  {
    global $argv;

    $this->RunCommon();
    if(count($argv) > 1)
    {
      if(array_key_exists($argv[1], $this->CommandLine))
      {
        $Command = $this->CommandLine[$argv[1]];
        $Output = call_user_func($Command['Callback'], $argv);
      } else $Output = 'Command "'.$argv[1].'" not supported.'."\n";
    } else $Output = 'No command was given as parameter'."\n";
    echo($Output);
  }

  function RegisterCommandLine($Name, $Callback)
  {
    $this->CommandLine[$Name] = array('Name' => $Name, 'Callback' => $Callback);
  }

  function RegisterPageBar($Name)
  {
    $this->Bars[$Name] = array();
  }

  function UnregisterPageBar($Name)
  {
    unset($this->Bars[$Name]);
  }

  function RegisterPageBarItem($BarName, $ItemName, $Callback)
  {
    $this->Bars[$BarName][$ItemName] = $Callback;
  }

  function RegisterPageHeader($Name, $Callback)
  {
    $this->PageHeaders[$Name] = $Callback;
  }
}

class PageMissing extends Page
{
  var $FullTitle = 'Stránka nenalezena';
  var $ShortTitle = 'Stránka nenalezena';

  function __construct($System)
  {
    parent::__construct($System);
    $this->ParentClass = 'PagePortal';
  }

  function Show()
  {
    Header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found');
    return('<h3 align="center">Požadovaná stránka neexistuje.</h3>');
  }
}
